%
% SLK EXCHANGE SIMULATIONS
%
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% Patient Generator Function for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
function [Patient_list,Comp_K,Comp_L]=...
    createpatientDICH_SLK_exchange_KOREA_3(n_KA,n_SLK,n_LA, ...
    ABO_distp, ABO_distd, PRA_dist, PRA_values, Female_ratio, ...
    Weight_dist, Height_dist, Leftlobe_dist, threshold_L)
%patients 1,...,n_KA are only kidney
% n_KA+1,...,n_KA+n_SLK are kidney+liver
% n_KA+n_SLK+1,...,n are only liver seekers

n=n_KA+n_SLK+n_LA;
n_K=n_KA+n_SLK;
%n_L=n_LA+n_SLK;


%ABOComp_K=zeros(n,n);     %ABOComp(i,j)=1 menas that patient i and donor j are ABOComple
%ABOComp_L=zeros(n,n);     %ABOComp(i,j)=1 menas that patient i and donor j are ABOComple

Comp_K=zeros(n,n);        %1st donor -- Kidney Donor Comp(i,j)=1 means that patient i and donor j are ABOComple and  negative crossmatch
Comp_L=zeros(n,n);        %2nd donor -- Liver Donor

totalgen=0;             %how many trials were needed to generate the n pair list

%1...n_KA: kdiney alone patients, n_KA+1,...,nK: SLK patients, n_K+1,...,n: liver alone patients

Patient_list=[];
for i=1:n
    Bloodt=zeros(1,3);
    Female=zeros(1,3);
    L_vo=[];
    for kk=1:3
        if kk==1
            if i<=n_KA
                fr=Female_ratio(1);
            elseif i<=n_K
                fr=Female_ratio(2);
            else
                fr=Female_ratio(3);
            end;
        elseif kk==2
            fr=Female_ratio(4);
        else
            fr=Female_ratio(5);
        end;
        
        totalgen=totalgen+1;
        Newcome=rand(7,1); %1,2,3 blood types of donors, 4 PRA, 5,6,7 weights
        z=1;
        if kk==1
            ABOdist=ABO_distp;
        else
            ABOdist=ABO_distd;
        end;
        while Newcome(kk)>ABOdist(z)
            z=z+1;
        end;
        Bloodt(kk)=z;
        Female(kk)=(Newcome(4+kk)<fr);
               
        if kk~=2
            if Female(kk)>0
                %Wes=Weight_dist(:,2);
                Hes=Height_dist(:,2);
            else
                %Wes=Weight_dist(:,1);
                Hes=Height_dist(:,1);
            end;
            
            %weight=trunrndn([1 1],Wes(1),Wes(2),Wes(3),Wes(4));
            height=trunrndn([1 1],Hes(1),Hes(2),Hes(3),Hes(4));
            if Female(kk)
                weight=exp(3.49+1.45*log(height));
            else
                weight=exp(3.28+1.92*log(height));
            end;
            BSA=(weight*height)^(.5)/6;
            livvol=(893.485*BSA-439.169)/1000;
            if kk==3
                leftlobe=trunrndn([1 1],Leftlobe_dist(1),Leftlobe_dist(2),Leftlobe_dist(3),Leftlobe_dist(4));
                livvol=leftlobe*livvol;
            end;
            L_vo=[L_vo livvol];
        end;
        if kk==1
            z=1;
            while Newcome(4)>PRA_dist(z)
                z=z+1;
            end;
            PRA=PRA_values(z);
        end;
        
        
    end;
    newpat=[Bloodt L_vo PRA Female];
    Patient_list=[Patient_list;newpat];
end;
Cross=rand(n,n);
%sth=sizethreshold;
for i=1:n                %patient
    for j=1:n  %donors
        ABOc_K=0;
        ABOc_L=0;
        if Patient_list(j,2)==1
            ABOc_K=1;
        elseif Patient_list(i,1)==4
            ABOc_K=1;
        elseif Patient_list(j,2)==Patient_list(i,1)
            ABOc_K=1;
        end;
        if Patient_list(j,3)==1
            ABOc_L=1;
        elseif Patient_list(i,1)==4
            ABOc_L=1;
        elseif Patient_list(j,3)==Patient_list(i,1)
            ABOc_L=1;
        end;
        if i<=n_K
            Comp_K(i,j)=ABOc_K & (Patient_list(i,6)<=Cross(i,j));
        end;
        if i>n_KA
            Comp_L(i,j)=ABOc_L & (threshold_L*Patient_list(i,4)<=Patient_list(j,5));
        end;
    end;   %for j
end;    %for i
SelfComp=zeros(n,1);
for i=1:n
    if i<=n_KA
        SelfComp(i)=Comp_K(i,i);
    elseif i<=n_K
        SelfComp(i)=Comp_K(i,i)*Comp_L(i,i);
    else
        SelfComp(i)=Comp_L(i,i);
    end;
    
end;
Patient_list=[Patient_list SelfComp];
