%
% DUAL-GRAFT LIVER EXCHANGE SIMULATIONS
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% Patient Generator Function for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
function [Patient_list, Cd1, Cd2, ABOcd1, ABOcd2, TTlist]=...
    createpatientDICH_dual_lobe_LIVER_exchange_KOREA_2(n, ABOdistp, ABOdistd, femp_ratio, femd_ratio, ...
    Weight_dist, Height_dist, Leftlobe_dist, threshold);

ABOcd1=zeros(n,n);     %ABOComp(i,j)=1 menas that patient i and donor j are ABOComple
ABOcd2=zeros(n,n);     %ABOComp(i,j)=1 menas that patient i and donor j are ABOComple
Cd1=zeros(n,n);        %Comp(i,j)=1 means that patient i and donor j are ABOComple and  negative crossmatch
Cd2=zeros(n,n);

%totalgen=0;             %how many trials were needed to generate the n pair list
Patient_list=[];
TTlist=zeros(4,4,4);
for i=1:n
    %totalgen=totalgen+1;
    Newcome=rand(7,1);
    Bloodt=zeros(1,3);
    L_vo=zeros(1,3);
    Female=zeros(1,3);
    for kk=1:3
        z=1;
        if kk==1
            ABOdist=ABOdistp;
            Female(kk)=(Newcome(4+kk)<femp_ratio);
        else
            ABOdist=ABOdistd;
            Female(kk)=(Newcome(4+kk)<femd_ratio);
        end;
        while Newcome(kk)>ABOdist(z)
            z=z+1;
        end;
        Bloodt(kk)=z;
        
        
        if Female(kk)
            %Wes=Weight_dist(:,2);
            Hes=Height_dist(:,2);
        else
            %Wes=Weight_dist(:,1);
            Hes=Height_dist(:,1);
        end;
        
        %weight=trunrndn([1 1],Wes(1),Wes(2),Wes(3),Wes(4));
        height=trunrndn([1 1],Hes(1),Hes(2),Hes(3),Hes(4));
        
        if Female(kk)
                weight=exp(3.49+1.45*log(height));
            else
                weight=exp(3.28+1.92*log(height));
            end;
        BSA=(weight*height*100/3600)^(.5);
        livvol=(893.485*BSA-439.169)/1000;
        if kk>1
            leftlobe=trunrndn([1 1],Leftlobe_dist(1),Leftlobe_dist(2),Leftlobe_dist(3),Leftlobe_dist(4));
            livvol=leftlobe*livvol;
        end;
        L_vo(kk)=livvol;
    end;
    New_pat=[Bloodt L_vo Female]; %1x3 blood types, 1x3 liv vols, 1x3 female? or not
    Patient_list=[Patient_list; New_pat];
    pb=Bloodt(1);
    db1=Bloodt(2);
    db2=Bloodt(3);
    if db2<db1
        btemp=db2;
        db2=db1;
        db1=btemp;
    end;
    TTlist(pb,db1,db2)=...
        TTlist(pb,db1,db2)+1;
end;

for i=1:n                %patient
    for j=1:n
        %donors
        if Patient_list(j,2)==1
            ABOcd1(i,j)=1;
        elseif Patient_list(i,1)==4
            ABOcd1(i,j)=1;
        elseif Patient_list(j,2)==Patient_list(i,1)
            ABOcd1(i,j)=1;
        end;
        if Patient_list(j,3)==1
            ABOcd2(i,j)=1;
        elseif Patient_list(i,1)==4
            ABOcd2(i,j)=1;
        elseif Patient_list(j,3)==Patient_list(i,1)
            ABOcd2(i,j)=1;
        end;
        
        Cd1(i,j)=ABOcd1(i,j)&(threshold*Patient_list(i,4)<=Patient_list(j,5));
        Cd2(i,j)=ABOcd2(i,j)&(threshold*Patient_list(i,4)<=Patient_list(j,6));
        
    end;   %for j
    
end;    %for i
Sc_1d=zeros(n,1);
Sc_2d=zeros(n,1);
for i=1:n
    Sc_1d(i)=(Cd1(i,i)|Cd2(i,i));
    Sc_2d(i)=(~Sc_1d(i))&ABOcd1(i,i)&ABOcd2(i,i)&(threshold*Patient_list(i,4)<=(Patient_list(i,6)+Patient_list(i,5)));
end;
Patient_list=[Patient_list Sc_1d Sc_2d];

