%
% DUAL-GRAFT LIVER EXCHANGE SIMULATIONS
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% Main Simulation File for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
% this version uses CPLEX of IBM for integer programming
%

diary off;
clear
fnamestr='dual-liver-KOREA-out-';

max_sim=500;
%sample sizes
Pops(1)=50;
Pops(2)=100;
Pops(3)=250;

ini_pop=1;
max_pop=3;
denomstd=1;
cplex_true=true;

ini_cyc_size=1; %unrestricted cycle size is denoted by 1
unrestricted_true=(ini_cyc_size==1);

max_cyc_size=3; %maximum cycle size in k-way, at most 4 currently

set(0,'RecursionLimit',1000);
max_rate=1; %fix at 1 for dual lobe

dstr=date;
str1=int2str(max_pop);
str2=int2str(max_rate);
mcsstr=int2str(max_cyc_size);
dname=strcat(fnamestr,'diary-',dstr,'-',mcsstr,'-',str1,'-',str2,'.txt');
size_comp_true=1;
diary(dname);

cp=3; %variation of agent sizes
cd=3;

ABOdistp=csvread('ABOdist_KOREA.csv')

ABOdistd=csvread('ABOdist_KOREA.csv')
Weight_dist=csvread('weightdist_KOREA.csv') %first column male, second column female: rows mean, std, min, max
Height_dist=csvread('heightdist_KOREA.csv') %first column male, second column female: rows mean, std, min, max
Leftlobe_dist=csvread('donorleftlobedist_KOREA.csv') %percentage of total volume rows mean, std, min, max

femp_ratio=0.346
femd_ratio=0.266

size_rate=0.4;


ansavr=[];
ansstd=[];


for num_pop=ini_pop:max_pop
    for num_rate=1:max_rate;
        threshold=size_rate(num_rate);
        
        seed=10*9*8*7*6*5*4*3*2;
   
        n=Pops(num_pop);
        
        fprintf('Running for population size %i; Running for size threshold: %4.1f \n', n, threshold);
        
        stats=[];
 
        for num_sim=1:max_sim
            fprintf('Pop: %4i; Sim: %4i; ', n, num_sim);
 
            seed0=seed*num_sim+1;
            S=rng;
            rng(seed0,'twister');
            tic;
            [Patient_list, Cd1, Cd2, ABOcd1, ABOcd2, TTlist]=...
                createpatientDICH_dual_lobe_LIVER_exchange_KOREA_2(n, ABOdistp,...
                ABOdistd, femp_ratio, femd_ratio, ...
                Weight_dist, Height_dist, Leftlobe_dist, threshold);
            rng(S);
            t=toc;
            tic;
  
            fprintf('Pop generated, time: %8.2f; ',t);
            Livp=Patient_list(:,4); %liver volume of patient
            Livd1=Patient_list(:,5); %graft volume of donor 1
            Livd2=Patient_list(:,6); %graft volume of donor 2
            
            clast=size(Patient_list,2);
            Sc_1D=Patient_list(:,clast-1); %1 donor self compatible patients
            
            Cd1_1D=Cd1((~Sc_1D),(~Sc_1D)');
            ABOcd1_1D=ABOcd1((~Sc_1D),(~Sc_1D)');
            
            Cd2_1D=Cd2((~Sc_1D),(~Sc_1D)');
            ABOcd2_1D=ABOcd2((~Sc_1D),(~Sc_1D)');
            
            Livd1_1D=Livd1(~Sc_1D);
            Livd2_1D=Livd2(~Sc_1D);
            Livp_1D=Livp(~Sc_1D);
            
            Plist_1D=Patient_list(~Sc_1D,:);
            
            [pid,ss]=find(Sc_1D);
            
            selfnum_1D=sum(Sc_1D);
            sfemp_1D=0;
            sfemd_1D=0;
            for id=1:selfnum_1D
                p=pid(id);
                sfemp_1D=sfemp_1D+Patient_list(p,7);
                if Cd1(p,1)&Cd1(p,2)
                    if Patient_list(p,5)>Patient_list(p,6)
                        sfemd_1D=sfemd_1D+Patient_list(p,8);
                    else
                        sfemd_1D=sfemd_1D+Patient_list(p,9);
                    end;
                elseif Cd1(p,1)
                    sfemd_1D=sfemd_1D+Patient_list(p,8);
                else
                    sfemd_1D=sfemd_1D+Patient_list(p,9);
                end;
            end;
    
            
            n_1D=size(Cd2_1D,1);
            
            %1 donor exchange first
            
            C_y=[Cd1_1D Cd2_1D; Cd1_1D Cd2_1D];
            Same_pat_y=[(1:n_1D)' (n_1D+1:2*n_1D)'];
            n_y=2*n_1D;
            We_y=ones(n_y,1);
            NDD_y=[];
            XNDD_y=zeros(n_y,1);
            weight_true=false;
            coef1_y=1;
            coef2_y=1;
            
            results=[n];
            gen_num=1;
            for num_cyc_size=ini_cyc_size:max_cyc_size
                
                exnum_x=0;
                Mu_x=zeros(n_y,1);
                if num_cyc_size>1
                    
                    if num_cyc_size==2
                        [num_cyc_1D, Cyc_size_1D, Cycles_1D, In_cyc_1D, W_cyc_1D]=...
                            wfind_cyc_sizeupto4_one_lobe_liver_exchange(C_y, We_y, NDD_y,...
                            max_cyc_size, weight_true); %the last bit says find weights (1) or not (0)
                        t=toc;
                        tic;
                        fprintf('\n 1-Donor cycles, time: %8.2f;',t);
                    end;
                    fprintf('\n %i-way        ', num_cyc_size);
                    [Mu_x,exnum_x]=...
                        wMAX_K_WAY_one_lobe_liver_exchange(n_y, C_y, We_y, num_cyc_1D,...
                        Cyc_size_1D, Cycles_1D, In_cyc_1D, W_cyc_1D, num_cyc_size,...
                        Same_pat_y, gen_num, weight_true, cplex_true);
                else
                    fprintf('\n Unrestricted ');
                    [Mu_x,exnum_x]=...
                        wMAX_UNR_one_lobe_liver_exchange(n_y, C_y, We_y, Same_pat_y, NDD_y,...
                        XNDD_y, gen_num, coef1_y, coef2_y, weight_true, cplex_true);
                end;
   
                t=toc;
                tic;
                fprintf('1-Donor exchange, time: %8.2f; ',t);
                [pid,did]=find(Mu_x);
                efemp_1D=0;
                efemd_1D=0;
                nid=size(pid,1);
                for id=1:nid
                    p=pid(id);
                    if p>n_1D
                        d=2;
                        p=p-n_1D;
                    else
                        d=1;
                    end;
                    efemp_1D=efemp_1D+Plist_1D(p,7);
                    efemd_1D=efemd_1D+Plist_1D(p,7+d);
                end;
   
                
                Unmatched_1D=~((Mu_x(1:n_1D,1)>0)|(Mu_x(n_1D+1:n_y,1)>0));
                
                Sc_2D=Plist_1D(:,clast);
                Avail_2D=(~Sc_2D)&Unmatched_1D;
                Plist_2D=Plist_1D(Avail_2D,:);
                exnum_1D=sum(~Unmatched_1D);
                
                ABOcd1_2D=ABOcd1_1D(Avail_2D,Avail_2D');
                ABOcd2_2D=ABOcd2_1D(Avail_2D,Avail_2D');
                n_2D=size(ABOcd1_2D,1);
                Livd1_2D=Livd1_1D(Avail_2D);
                Livd2_2D=Livd2_1D(Avail_2D);
                Livp_2D=Livp_1D(Avail_2D);
                
                Livd_2D=[];
                for ii=1:n_2D
                    Livd_2D=[Livd_2D; Livd1_2D(ii); Livd2_2D(ii)];
                end;
                
                selfnum_2D=sum(Sc_2D & Unmatched_1D);
                
                [pid,ss]=find(Sc_2D & Unmatched_1D);
                sfemp_2D=0;
                sfemd_2D=0;
                for id=1:selfnum_2D
                    p=pid(id);
                    sfemp_2D= sfemp_2D+ Plist_1D(p,7);
                    sfemd_2D= sfemd_2D+ (Plist_1D(p,8)+ Plist_1D(p,9))/2;
                end;
                
                %2 donor exchange next
                
                Ept=zeros(1,4);
                Apt=zeros(1,4);
                Spt=zeros(1,4);
  
                Same_pat=[];
     
                
                if num_cyc_size==1
                    C_c=[];
                    for ii=1:n_2D
                        C_c=[C_c ABOcd1_2D(:,ii) ABOcd2_2D(:,ii)];
                    end;
                    [Matched_2D, exnum_2D, Mu_2D, E_inout, E_out, E_donor, Edges, C_f, X]=...
                        wMAX_UNR_dual_lobe_liver_exchange(C_c, Livd_2D, Livp_2D, threshold, cplex_true);
                    t=toc;
                    tic;
                    fprintf('2-Donor exchange, time: %8.2f; ',t);
                else
    
                    [num_cyc_2D, Cyc_size_2D, Cycles_2D, In_cyc_2D, W_cyc_2D]=...
                        wfind_cyc_sizeupto5_dual_lobe_liver_exchange(ABOcd1_2D,ABOcd2_2D,Livd_2D,Livp_2D,threshold,num_cyc_size,cplex_true);
                    t=toc;
                    tic;
                    
                    fprintf('2-Donor cycles, time: %8.2f; ',t);
                    [Matched_2D,exnum_2D,Mu_2D]=wMAX_K_WAY_lung_exchange(n_2D,Cyc_size_2D,...
                        Cycles_2D,In_cyc_2D,W_cyc_2D,num_cyc_size,Same_pat,gen_num,cplex_true);
                    t=toc;
                    tic;
                    
                    fprintf('2-Donor exchange, time: %8.2f; ',t);
                end;
      
                
                [pid,did]=find(Matched_2D);
     
                efemp_2D=0;
                efemd_2D=0;
                nid=size(pid,1);
                for id=1:nid
                    p=pid(id);
                    efemp_2D= efemp_2D+ Plist_2D(p,7);
                    efemd_2D= efemd_2D+ (Plist_2D(p,8)+ Plist_2D(p,9))/2;
                end;
                r=[results -num_cyc_size  selfnum_1D exnum_1D selfnum_2D exnum_2D -num_cyc_size ...
                    sfemp_1D sfemd_1D efemp_1D efemd_1D sfemp_2D sfemd_2D efemp_2D efemd_2D -num_cyc_size];
                results=r;
                %t=toc;
                %tic;
                
            end; %num_cyc_size
            fprintf('\n');
            diary on;
            round(results)
            diary off;
            
            results=[results mean(Patient_list) std(Patient_list)];
  
            stats=[stats;results];
        end;
        str1=int2str(num_pop);
        str2=int2str(num_rate);
        
        outfilename=strcat(fnamestr,dstr,'-',mcsstr,'-',str1,'-',str2,'.csv');
        outfilename2 =strcat(fnamestr,'AVERAGE-',dstr,'-',mcsstr,'-',str1,'-',str2,'.csv');
        outfilename3 =strcat(fnamestr,'STD-',dstr,'-',mcsstr,'-',str1,'-',str2,'.csv');
        
        csvwrite(outfilename,stats);
       
        
        ansavr=[ansavr;mean(stats)];
        ansstd=[ansstd;std(stats)/denomstd];
        
        csvwrite(outfilename2,ansavr);
        ansavr
        csvwrite(outfilename3,ansstd);
        ansstd
    end;
end;
diary off;