%
% LUNG EXCHANGE SIMULATIONS
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% Main Simulation File for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
% this version uses CPLEX of IBM for integer programming
%


diary off;
clear

fnamestr='lung-JAPAN-out-';


max_sim=500;
%sample sizes
Pops(1)=10;
Pops(2)=20;
Pops(3)=50;
ini_pop=1;
max_pop=3;
denomstd=1;
cplex_true=true;
unrestricted_true=true;

max_cyc_size=5;  %maximum cycle size in k-way


ini_size_comp=1;
max_size_comp=1;%1st considers size compatibility and 2nd one doesnot, 2 or 1


set(0,'RecursionLimit',1000);
str1=int2str(max_pop);
str2=int2str(max_size_comp);
dstr=date;
mcsstr=int2str(max_cyc_size);
dname=strcat(fnamestr,'diary-',dstr,'-',mcsstr,'-',str1,'-',str2,'.txt');
diary(dname);

cdev=3;

ABOdist_p=csvread('ABOdist_JAPAN.csv')
ABOdist_d=csvread('ABOdist_JAPAN.csv')
Hum_we_d=csvread('weightdist_JAPAN.csv');
hum_we_mu=Hum_we_d(1)
hum_we_sig=Hum_we_d(2)
hum_we_min=hum_we_mu-cdev*hum_we_sig
hum_we_max=hum_we_mu+cdev*hum_we_sig

ansavr=[];
ansstd=[];


for num_pop=ini_pop:max_pop
    for num_size_comp=ini_size_comp:max_size_comp;
        if num_size_comp==2
            size_comp_true=false;
        else
            size_comp_true=true;
        end;
        
        
        seed=9*8*7*6*5*4*3*2;
        seed2=seed*10;
        n=Pops(num_pop);
        
        'Running for population size'
        n
        'Running for size compatibility'
        size_comp_true
        
        
        
        stats=[];
        plist=[];
        Match_list=[];
        for num_sim=1:max_sim
            
            seed0=seed*num_sim+1;
            S=rand('state');
            rand('state',seed0);
            
            
            [Patient_list,Comp1,Comp2,TTlist]=...
                createpatientDICH_LUNG_exchange_JAPAN_1(n,ABOdist_p,...
                ABOdist_d,hum_we_mu,hum_we_sig,hum_we_min,hum_we_max,size_comp_true);
            
            
            rand('state',S);
            clast=size(Patient_list,2);
            SelfComp=logical(Patient_list(:,clast));
            
            
            Comp1_ns=Comp1((~SelfComp),:);
            Comp1_ns=Comp1_ns(:,(~SelfComp)');
            
            Comp2_ns=Comp2((~SelfComp),:);
            Comp2_ns=Comp2_ns(:,(~SelfComp)');
            
            Pnself=Patient_list(~SelfComp,:);
            n_ns=size(Comp2_ns,1);
            Pself=Patient_list(SelfComp,:);
            
            We_ns=ones(n_ns,1);
            selfnum=sum(SelfComp);
            Same_pat=[];
            NDD=[];
            Ept=zeros(1,4);
            Apt=zeros(1,4);
            Spt=zeros(1,4);
            
            XNDD=zeros(n_ns,1);
            gennum=1;
            
            
            exnum_unr=0;
            Mu_unr=zeros(n_ns,2);
            Matched_unr=zeros(n_ns,1);
            
            
            tic;
            if unrestricted_true
                
                fprintf('Unr...');
                
                [Matched_unr,exnum_unr,Mu_unr,E_inout, E_out, E_donor, Edges, C_flow,X]=...
                    wMAX_UNR_lung_exchange(n_ns,Comp1_ns,Comp2_ns,cplex_true);
                
                
                ct=toc;
                tic;
                fprintf('time: %6.2f; ',ct);
                
            end;
            fprintf('Cycles...');
            
            [num_c,Cyc_size,Cycles,In_cyc,W_cyc]=...
                wfind_cyc_sizeupto5_lung_exchange_fast(n_ns,Comp1_ns,Comp2_ns,We_ns,max_cyc_size);
            ct=toc;
            tic;
            fprintf('time: %6.2f; ',ct);
            
            
            
            
            exnum_5way=0;
            exnum_4way=0;
            exnum_3way=0;
            exnum_2way=0;
            
           
            
            Matched_5way=zeros(n_ns,1);
            Matched_4way=zeros(n_ns,1);
            Matched_3way=zeros(n_ns,1);
            Matched_2way=zeros(n_ns,1);
            
            Mu_5way=zeros(n_ns,1);
            Mu_4way=zeros(n_ns,1);
            Mu_3way=zeros(n_ns,1);
            Mu_2way=zeros(n_ns,1);
            
            if max_cyc_size<=6
                if max_cyc_size>=5
                    fprintf('5-way...');
                    [Matched_5way,exnum_5way,Mu_5way]=wMAX_K_WAY_lung_exchange(n_ns,Cyc_size,...
                        Cycles,In_cyc,W_cyc,5,Same_pat,gennum,cplex_true);
                    ct=toc;
                    tic;
                    
                    fprintf('time: %6.2f; ',ct);
                end;
                if max_cyc_size>=4
                    fprintf('4-way...');
                    [Matched_4way,exnum_4way,Mu_4way]=wMAX_K_WAY_lung_exchange(n_ns,Cyc_size,...
                        Cycles,In_cyc,W_cyc,4,Same_pat,gennum,cplex_true);
                    ct=toc;
                    tic;
                    
                    fprintf('time: %6.2f; ',ct);
                end;
                if max_cyc_size>=3
                    fprintf('3-way...');
                    [Matched_3way,exnum_3way,Mu_3way]=wMAX_K_WAY_lung_exchange(n_ns,Cyc_size,...
                        Cycles,In_cyc,W_cyc,3,Same_pat,gennum,cplex_true);
                    ct=toc;
                    tic;
                    fprintf('time: %6.2f; ',ct);
        
                end;
                if max_cyc_size>=2
                    fprintf('2-way...');
                    [Matched_2way,exnum_2way,Mu_2way]=wMAX_K_WAY_lung_exchange(n_ns,Cyc_size,...
                        Cycles,In_cyc,W_cyc,2,Same_pat,gennum,cplex_true);
                    ct=toc;
                    tic;
                    fprintf('time: %6.2f; ',ct);
                    
                end;
                fprintf('\n');
                
            end;
            Mxpt=[];
            Match_list=[];
            TAK=-1*ones(1,size(Patient_list,2));
            for anal=1:5
                if anal==1
                    Mu=Mu_2way;
                elseif anal==2
                    Mu=Mu_3way;
                elseif anal==3
                    Mu=Mu_4way;
                elseif anal==4
                    Mu=Mu_5way;
                else
                    Mu=Mu_unr(:,1);
                end;
                Matched=Pnself(Mu>0,:);
                effs=size(Matched,1);
                
                if ~isempty(Matched)
                    Match_list=[Match_list mean(Matched) std(Matched)];
                else
                    Match_list=[Match_list TAK TAK];
                end;
                
                
                
            end;
            results=[num_pop num_size_comp num_sim n selfnum exnum_2way exnum_3way exnum_4way exnum_5way ...
                exnum_unr  mean(Patient_list) std(Patient_list) mean(plist) ...
                std(plist) Match_list];
            [num_pop num_size_comp num_sim n selfnum exnum_2way exnum_3way exnum_4way exnum_5way exnum_unr]
            
            stats=[stats;results];
        end;
        str1=int2str(num_pop);
        str2=int2str(num_size_comp);
        outfilename=strcat(fnamestr,dstr,'-',mcsstr,'-',str1,'-',str2,'.csv');
        outfilename2 =strcat(fnamestr,'AVERAGE-',dstr,'-',mcsstr,'-',str1,'-',str2,'.csv');
        outfilename3 =strcat(fnamestr,'STD-',dstr,'-',mcsstr,'-',str1,'-',str2,'.csv');
        csvwrite(outfilename,stats);
        ansavr=[ansavr;mean(stats)];
        ansstd=[ansstd;std(stats)/denomstd];
        csvwrite(outfilename2,ansavr);
        csvwrite(outfilename3,ansstd);
    end;
end;
diary off;