%
% SLK EXCHANGE SIMULATIONS
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% Main Simulation File for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
% this version uses CPLEX of IBM for integer programming
%abbreviations:
%KA - kidney alone
%LA - liver alone
%SLK - simultaneous liver kidney
%Only two-way exchange

diary off;
clear
fnamestr='SLK-KOREA-out-';
max_sim=500;

ini_treat=1; %treatment numbers 1: separate pools, 2: joint pools
max_treat=2;

%sample sizes
Pops=zeros(3,1);

%kidney pool population sizes
Pops(1)=250;
Pops(2)=500;
Pops(3)=1000;

ini_pop=1;
max_pop=3;

ini_perc=1;
max_perc=2;

denomstd=1;

cplex_true=true;
size_comp_true=true;
unrestricted_true=false; %alsways set false


num_cyc_size=3; %maximum patient number in integrated KA, SLK, LA in each exchange

set(0,'RecursionLimit',1000);




dstr=date;
str1=int2str(max_pop);
str2=int2str(max_treat);
str3=int2str(max_perc);
mcsstr=int2str(num_cyc_size);

dname=strcat(fnamestr,'diary-',dstr,'-',mcsstr,'-',str1,'-',str2,'.txt');

diary(dname);

ABO_distp=csvread('ABOdist_KOREA.csv')
ABO_distd=csvread('ABOdist_KOREA.csv')

%rng=[1 1 1 1];
%rng2=[1 1 4 1];

perc=csvread('SLKratios_KOREA_3.csv') 

threshold_L=0.4

Female_ratio=csvread('femaleratio_KOREA_KL.csv')
weight_true=false;

Weight_dist=csvread('weightdist_KOREA.csv')
Height_dist=csvread('heightdist_KOREA.csv')

PRAdistall=csvread('PRAdist_3.csv')
PRA_dist=PRAdistall(:,1);
PRA_values=PRAdistall(:,2);
Leftlobe_dist=csvread('donorleftlobedist_KOREA.csv')


ansavr=[];
ansstd=[];


for num_pop=ini_pop:max_pop
    
    for num_perc=1:max_perc
        n=Pops(num_pop);
        n_SLK=round(n*perc(num_perc,1));
        n_LA=round(n*perc(num_perc,2));
        n_KA=n-n_LA-n_SLK;
        n_K=n_KA+n_SLK;
        n_L=n_LA+n_SLK;
        Pats=[1:n]';
        
        for num_treat=ini_treat:max_treat;
            
            
            seed=10*9*8*7*6*5*4*3*2;
            
            
            
            'Running for population sizes for KA, SLK, LA'
            [n_KA n_SLK n_LA]
            
            'Running for treatment'
            if num_treat==1
                'KA, SLK, LA, separate'
            else
                'KA, SLK, LA together'
            end;
            
            
            stats=[];
            plist=[];
            Matchlist=[];
            for num_sim=1:max_sim
             
                seed0=seed*num_sim+1;
                S=rng;
                rng(seed0,'twister');
                tic;
                
                spouseratio=0.0;
                [Plist,Comp_K,Comp_L]=...
                    createpatientDICH_SLK_exchange_KOREA_3(n_KA,n_SLK, n_LA,...
                    ABO_distp, ABO_distd, PRA_dist, PRA_values, Female_ratio, ...
                    Weight_dist, Height_dist, Leftlobe_dist, threshold_L);
                
                
                rng(S);
                clast=size(Plist,2);
                Sc=logical(Plist(:,clast));
                

                Sc_KA=Sc&(Pats<=n_KA);
                Sc_SLK=Sc&((Pats>n_KA)&(Pats<=n_K));
                Sc_LA=Sc&(Pats>n_K);
                
                Nsc=~Sc;
                Nsc_KA=(Nsc)&(Pats<=n_KA);
                Nsc_SLK=(Nsc)&((Pats>n_KA)&(Pats<=n_K));
                Nsc_LA=(Nsc)&(Pats>n_K);
                
                
                
              
                
                selfnum=sum(Sc);
                selfnum_KA=sum(Sc_KA);
                selfnum_LA=sum(Sc_LA);
                selfnum_SLK=sum(Sc_SLK);
                
                Comp_K_ns=Comp_K((Nsc),:);
                Comp_K_ns=Comp_K_ns(:,(Nsc)');
                
                Comp_L_ns=Comp_L((Nsc),:);
                Comp_L_ns=Comp_L_ns(:,(Nsc)');
                
               
                n_ns=size(Comp_L_ns,1);
                
                
                
               
                Same_pat=[];
                NDD=[];
                gen_num=1;
               
                Ept=zeros(1,4);
                Apt=zeros(1,4);
                Spt=zeros(1,4);
                
                %if n2>0
                XNDD=zeros(n_ns,1);
                gennum=1;
                
                Comp_K_KA_ns=Comp_K(Nsc_KA,Nsc_KA');
                Comp_L_KA_ns=Comp_L(Nsc_KA,Nsc_KA');
                
                Comp_K_SLK_ns=Comp_K(Nsc_SLK,Nsc_SLK');
                Comp_L_SLK_ns=Comp_L(Nsc_SLK,Nsc_SLK');
                
                Comp_L_LA_ns=Comp_L(Nsc_LA,Nsc_LA');
                Comp_K_LA_ns=Comp_K(Nsc_LA,Nsc_LA');
                
                n_KA_ns=size(Comp_K_KA_ns,1);
                n_SLK_ns=size(Comp_K_SLK_ns,1);
                n_LA_ns=size(Comp_K_LA_ns,1);
                
                We_ns=ones(n_ns,1);
                We_KA_ns=ones(n_KA_ns,1);
                We_SLK_ns=ones(n_SLK_ns,1);
                We_LA_ns=ones(n_LA_ns,1);
                
                
                
                exnum=0;
                exnum_KA=0;
                exnum_SLK=0;
                exnum_LA=0;
                
                Matched=zeros(n_ns,1);
                Matched_KA=zeros(n_KA_ns,1);
                Matched_SLK=zeros(n_SLK_ns,1);
                Matched_LA=zeros(n_LA_ns,1);
                diary off;
                if num_treat==1
                   
                    [num_cyc_KA,Cyc_size_KA,Cycles_KA,In_cyc_KA,W_cyc_KA]=...
                        wfind_cyc_size2_SLK_exchange(n_KA_ns,0,0,Comp_K_KA_ns,Comp_L_KA_ns,We_KA_ns);
                    
                    [num_cyc_SLK,Cyc_size_SLK,Cycles_SLK,In_cyc_SLK,W_cyc_SLK]=...
                        wfind_cyc_size2_SLK_exchange(0,n_SLK_ns,0,Comp_K_SLK_ns,Comp_L_SLK_ns,We_SLK_ns);
                    [num_cyc_LA,Cyc_size_LA,Cycles_LA,In_cyc_LA,W_cyc_LA]=...
                        wfind_cyc_size2_SLK_exchange(0,0,n_LA_ns,Comp_K_LA_ns,Comp_L_LA_ns,We_LA_ns);
                    
                    
                    %fprintf('Calculating 2-way Optimized Match ...');
                    
                    [Matched_KA,exnum_KA,Mu_KA]=...                    
                        wMAX_K_WAY_SLK_exchange(n_KA_ns,Cyc_size_KA,Cycles_KA,In_cyc_KA,...
                        W_cyc_KA,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                    [Matched_SLK,exnum_SLK,Mu_SLK]=...                    
                        wMAX_K_WAY_SLK_exchange(n_SLK_ns,Cyc_size_SLK,Cycles_SLK,In_cyc_SLK,...
                        W_cyc_SLK,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                    
                    [Matched_LA,exnum_LA,Mu_LA]=...                    
                        wMAX_K_WAY_SLK_exchange(n_LA_ns,Cyc_size_LA,Cycles_LA,In_cyc_LA,...
                        W_cyc_LA,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                    exnum=exnum_KA+exnum_LA+exnum_SLK;
                elseif num_treat==2
   
                    [num_cyc,Cyc_size,Cycles,In_cyc,W_cyc]=...
                        wfind_cyc_size2_SLK_exchange(n_KA_ns,n_SLK_ns,n_LA_ns,Comp_K_ns,Comp_L_ns,We_ns);
                    
                    %fprintf('Calculating 2-way Optimized Match ...');
                    [Matched,exnum,Mu]=...                    
                        wMAX_K_WAY_SLK_exchange(n_ns,Cyc_size,Cycles,In_cyc,...
                        W_cyc,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                    %fprintf('Done \n');
                    exnum_KA=sum(Matched(1:n_KA_ns));
                    exnum_SLK=sum(Matched(n_KA_ns+1:n_KA_ns+n_SLK_ns));
                    exnum_LA=sum(Matched(n_KA_ns+n_SLK_ns+1:n_ns));
                   
                end;
                
                
                diary on;
                results=[num_pop num_perc num_treat num_sim -1 n n_KA n_SLK n_LA  -2 ...
                    selfnum selfnum_KA selfnum_SLK selfnum_LA -3 exnum exnum_KA ...
                    exnum_SLK exnum_LA -4]
                diary off;
                results=[results   mean(Plist) std(Plist)];
                
                stats=[stats;results];
            end; %num_sim
            
            str1=int2str(num_pop);
            str2=int2str(num_treat);
            str3=int2str(num_perc);
            
            outfilename=strcat(fnamestr,dstr,'-',mcsstr,'-',str1,'-',str2,'-',str3,'.csv');
            outfilename2 =strcat(fnamestr,'AVERAGE-',dstr,'-',mcsstr,'-',str1,'-',str2,'-',str3,'.csv');
            outfilename3 =strcat(fnamestr,'STD-',dstr,'-',mcsstr,'-',str1,'-',str2,'-',str3,'.csv');
            
            csvwrite(outfilename,stats);
            %outfilename
            ansavr=[ansavr;mean(stats)]
            ansstd=[ansstd;std(stats)/denomstd];
            csvwrite(outfilename2,ansavr);
            csvwrite(outfilename3,ansstd);
        end; %treat_num
    end; %perc_num
end; %pop_num

diary off;