%
% LUNG EXCHANGE SIMULATIONS - DYNAMIC
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% Main Simulation File for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
% this version uses CPLEX of IBM for integer programming
%
diary off;
clear

fnamestr='dynamic-lung-JAPAN-out-';

max_sim=500;

%sample size
Pops(1)=200;

max_per=20;

ini_pop=1;
max_pop=1;

denomstd=1;
cplex_true=true;

max_cyc_size=3;  %maximum cycle size in k-way
ini_cyc_size=2;  %minimum cycle size in k-way 1 means unrestricted
unrestricted_true=(ini_cyc_size==1);

ini_size_comp=1;
max_size_comp=1;%1st considers size compatibility and 2nd one doesnot, 2 or 1


set(0,'RecursionLimit',1000);
str1=int2str(max_pop);
str2=int2str(max_size_comp);
str3=int2str(max_per);
dstr='';
mcsstr=int2str(max_cyc_size);
dname=strcat(fnamestr,'diary-',dstr,'-',mcsstr,'-',str3,'-',str1,'-',str2,'.txt');
diary(dname);

ABOdistp=csvread('ABOdist_JAPAN.csv')
ABOdistd=csvread('ABOdist_JAPAN.csv')
Weights=csvread('weightdist_JAPAN.csv');
we_mu=Weights(1)
we_sig=Weights(2)
we_min=Weights(3)
we_max=Weights(4)

ansavr=[];
ansstd=[];

for num_pop=ini_pop:max_pop
    for num_size_comp=ini_size_comp:max_size_comp;
        if num_size_comp==2
            size_comp_true=false;
        else
            size_comp_true=true;
        end;
        
        seed=9*8*7*6*5*4*3*2;
        seed2=seed*10;
        n=Pops(num_pop);
        
        'Running for population size'
        n
        'Running for size compatibility'
        size_comp_true
        'Number of Arrivals per period'
        arr=n/max_per
        'Number of periods'
        max_per
        
        stats=[];
        plist=[];
        MatchList=[];
        %Ns=zeros(max_sim,max_per);
        Selfnum=zeros(max_sim,max_per);
        Exnum_2way=zeros(max_sim,max_per);
        Exnum_3way=zeros(max_sim,max_per);
        Exnum_4way=zeros(max_sim,max_per);
        Exnum_5way=zeros(max_sim,max_per);
        Exnum_unr=zeros(max_sim,max_per);
        
        for num_sim=1:max_sim
            
            seed0=seed*num_sim+1;
            S=rand('state');
            rand('state',seed0);
                        
            [Patient_list,Comp1,Comp2,TTlist]=...
                createpatientDICH_LUNG_exchange_JAPAN_1(n,ABOdistp,...
                ABOdistd,we_mu,we_sig,we_min,we_max,size_comp_true);
                        
            rand('state',S);
            clast=size(Patient_list,2);
            Sc=logical(Patient_list(:,clast));
          
            selfnum=sum(Sc);
            Same_pat=[];
            NDD=[];
            Ept=zeros(1,4);
            Apt=zeros(1,4);
            Spt=zeros(1,4);
                     
            gen_num=1;
            
            Pats=[1:n]';
            
            exnum_2way=0;
            exnum_3way=0;
            exnum_4way=0;
            exnum_5way=0;
            exnum_unr=0;
            
            for num_cyc_size=ini_cyc_size:max_cyc_size
                Rem=zeros(n,1);
                tic;
                fprintf(' %i-way ', num_cyc_size);
                for num_per=1:max_per
                    Rem((num_per-1)*arr+1:num_per*arr,1)=1;
                    Rem=Rem&(~Sc);
                    
                    Comp1_ns=Comp1(Rem,Rem');
                    Comp2_ns=Comp2(Rem,Rem');
                    n_ns=size(Comp1_ns,1);
                    We_ns=ones(n_ns,1);
                    XNDD=zeros(n_ns,1);
                    P_ns=Pats(Rem,:);
                    %pause
                    exnum_kway=0;
                    Matched_kway=zeros(n_ns,1);
                    Mu_kway=zeros(n_ns,2);
                    selfnum_per=sum(Rem((num_per-1)*arr+1:num_per*arr)&Sc((num_per-1)*arr+1:num_per*arr));                   
                    Selfnum(num_sim,num_per)=selfnum_per;
                    if (num_cyc_size<6)&(num_cyc_size>1)
                        [num_cyc,Cyc_size,Cycles,In_cyc,W_cyc]=...
                            wfind_cyc_sizeupto5_lung_exchange_fast(n_ns,Comp1_ns,Comp2_ns,We_ns,num_cyc_size);
                        
                        [Matched_kway,exnum_kway,Mu_kway]=wMAX_K_WAY_lung_exchange(n_ns,Cyc_size,...
                            Cycles,In_cyc,W_cyc,num_cyc_size,Same_pat,gen_num,cplex_true);
                        if num_cyc_size==2
                            Exnum_2way(num_sim,num_per)=exnum_kway;
                        elseif num_cyc_size==3
                            Exnum_3way(num_sim,num_per)=exnum_kway; 
                        elseif num_cyc_size==4
                            Exnum_4way(num_sim,num_per)=exnum_kway; 
                        elseif num_cyc_size==5
                            Exnum_5way(num_sim,num_per)=exnum_kway; 
                        end;
                        
                    elseif (num_cyc_size==1)&unrestricted_true
                        [Matched_kway,exnum_kway, Mu_kway, Einout, Eout, Edonor, Edges, C,X]=...
                            wMAX_UNR_lung_exchange(n_ns,Comp1_ns,Comp2_ns,cplex_true);
                          Exnum_unr(num_sim,num_per)=exnum_kway;
                        
                    end;
                    
                    Matched=zeros(n,1);
                    for ii=1:n_ns
                        Matched(P_ns(ii))=(Matched_kway(ii)>0);
                    end;
                    Rem=Rem&(~Matched);
                    t=toc;
                    fprintf('%8.2f', t);
                end; %num_per
                
                if num_cyc_size==1
                    exnum_unr=sum(~Rem)-selfnum;
                  
                elseif num_cyc_size==2
                    exnum_2way=sum(~Rem)-selfnum;
                elseif num_cyc_size==3
                    exnum_3way=sum(~Rem)-selfnum;
                elseif num_cyc_size==4
                    exnum_4way=sum(~Rem)-selfnum;
                elseif num_cyc_size==5
                    exnum_5way=sum(~Rem)-selfnum;
                end;
                t=toc;
                fprintf(' time: %8.2f \n', t);
            end;
            results=[n num_sim selfnum exnum_2way exnum_3way exnum_4way exnum_5way ...
                exnum_unr  mean(Patient_list) std(Patient_list)];
            [n num_size_comp num_sim selfnum exnum_2way exnum_3way exnum_4way exnum_5way exnum_unr]
            
            stats=[stats;results];
            
        end;
        
        stats=[stats Selfnum Exnum_2way Exnum_3way Exnum_4way Exnum_5way Exnum_unr];
        str1=int2str(num_pop);
        str2=int2str(num_size_comp);
        outfilename=strcat(fnamestr,dstr,'-',mcsstr,'-',str3,'-',str1,'-',str2,'.csv');
        outfilename2 =strcat(fnamestr,'AVERAGE-',dstr,'-',mcsstr,'-',str3,'-',str1,'-',str2,'.csv');
        outfilename3 =strcat(fnamestr,'STD-',dstr,'-',mcsstr,'-',str3,'-',str1,'-',str2,'.csv');
        csvwrite(outfilename,stats);
        ansavr=[ansavr;mean(stats)];
        ansstd=[ansstd;std(stats)/denomstd];
        csvwrite(outfilename2,ansavr);
        csvwrite(outfilename3,ansstd);
    end;
end;
diary off;