%
% SLK EXCHANGE SIMULATIONS - DYNAMIC
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% Main Simulation File for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
% this version uses CPLEX of IBM for integer programming
%
% 
%abbreviations:
%KA - kidney alone
%LA - liver alone
%SLK - simultaneous liver kidney
%Only two-way exchange

diary off;
clear
fnamestr='dynamic-SLK-KOREA-out-';
max_sim=500;

ini_treat=1; %treatment numbers 1: separate pools, 2: joint pools
max_treat=2;

%sample sizes
Pops=zeros(1,1);
max_per=20; %period numbers

data_all=false; %set to true if you would like to get the WHOLE DATA'S EXCHANGE not period by period.

%kidney+liver pool population sizes
Pops(1)=2000; %do not change without adjusting ‘SLKarrival_dyn_KOREA_3.csv file


ini_pop=1;
max_pop=1;

ini_perc=1;
max_perc=2;

denomstd=1;

cplex_true=true;
size_comp_true=true;
unrestricted_true=false; %alsways set false



num_cyc_size=3; %maximum patient number in integrated KA, SLK, LA in each exchange

set(0,'RecursionLimit',1000);




dstr=date;
%str1=int2str(max_pop);
str2=int2str(max_treat);
if data_all
    str3='stat-dyn';
else
    str3='only-dyn';
end;
mcsstr=int2str(num_cyc_size);

dname=strcat(fnamestr,'diary-',dstr,'-',mcsstr,'-',str2,'-',str3,'.txt');

diary(dname);

ABO_distp=csvread('ABOdist_KOREA.csv')
ABO_distd=csvread('ABOdist_KOREA.csv')



perc=csvread('SLKratios_KOREA_3.csv') 

threshold_L=0.4

Female_ratio=csvread('femaleratio_KOREA_KL.csv')
weight_true=false;

Weight_dist=csvread('weightdist_KOREA.csv')
Height_dist=csvread('heightdist_KOREA.csv')
ARR0=csvread('SLKarrival_dyn_KOREA_3.csv')


PRAdistall=csvread('PRAdist_3.csv')
PRA_dist=PRAdistall(:,1);
PRA_values=PRAdistall(:,2);
Leftlobe_dist=csvread('donorleftlobedist_KOREA.csv')



ansavr=[];
ansstd=[];
Sep=-1*ones(max_sim,1);


for num_pop=ini_pop:max_pop
    
    for num_perc=ini_perc:max_perc
        n=Pops(num_pop);
        n_SLK=round(n*perc(num_perc,1));
        n_LA=round(n*perc(num_perc,2));
        n_KA=n-n_LA-n_SLK;
        
        n_K=n_KA+n_SLK;
        n_L=n_LA+n_SLK;
        
        
        
        arr_KA0=ARR0(4+num_perc,1:max_per)';
        arr_LA0=ARR0(2+num_perc,1:max_per)';
        
        arr_SLK0=ARR0(num_perc,1:max_per)';
        
        
        
        Pats=[1:n]';
        Pats_KA=[1:n_KA]';
        Pats_SLK=[n_KA+1:n_K]';
        Pats_LA=[n_K+1:n]';
        
        Arr_KA1=1;
        Arr_SLK1=n_KA+1;
        Arr_LA1=n_K+1;
        

        
        for num_treat=ini_treat:max_treat;
            
            
            seed=10*9*8*7*6*5*4*3*2;
            
      
            'Running for population sizes for KA, SLK, LA'
            [n_KA n_SLK n_LA]
           
            'Running for treatment'
            if num_treat==1
                'KA, SLK, LA, separate'
            else
                'KA, SLK, LA together'
            end;
            
            
            stats=[];
            
            results=[];
            Patnum_KA=zeros(max_sim, max_per);
            Patnum_LA=zeros(max_sim, max_per);
            Patnum_SLK=zeros(max_sim, max_per);
            Selfnum_SLK=zeros(max_sim, max_per);
            Selfnum_LA=zeros(max_sim, max_per);
            Selfnum_KA=zeros(max_sim, max_per);
            Exnum_KA=zeros(max_sim, max_per);
            Exnum_LA=zeros(max_sim, max_per);
            Exnum_SLK=zeros(max_sim, max_per);
            
            for num_sim=1:max_sim
     	        seed0=seed*num_sim+1;
                S=rng;
                rng(seed0,'twister');
                tic;
                Arr_SLK=Arr_SLK1;
                Arr_LA=Arr_LA1;
                Arr_KA=Arr_KA1;
                Perm_perSLK=randperm(max_per);
                Perm_perLA=randperm(max_per);
                Perm_perKA=randperm(max_per);
                arr_SLK=arr_SLK0(Perm_perSLK);
                arr_KA=arr_KA0(Perm_perKA);
                arr_LA=arr_LA0(Perm_perLA);
              
                for num_per=1:max_per
             
                    Arr_SLK=[Arr_SLK; Arr_SLK(num_per)+arr_SLK(num_per)];
                    Arr_LA=[Arr_LA; Arr_LA(num_per)+arr_LA(num_per)];
                    Arr_KA=[Arr_KA; Arr_KA(num_per)+arr_KA(num_per)];
                   
                end;
               
                spouseratio=0.0;
                [Plist,Comp_K,Comp_L]=...
                    createpatientDICH_SLK_exchange_KOREA_3(n_KA,n_SLK,n_LA,...
                    ABO_distp, ABO_distd, PRA_dist, PRA_values, Female_ratio, ...
                    Weight_dist, Height_dist, Leftlobe_dist, threshold_L);
                t=toc;
                tic;
                fprintf('Patient Pool!, time: %8.2f;', t);
                
                rng(S);
                clast=size(Plist,2);
                Sc=Plist(:,clast);
                
                
                Sc_KA=Sc&(Pats<=n_KA);
                Sc_SLK=Sc&((Pats>n_KA)&(Pats<=n_K));
                Sc_LA=Sc&(Pats>n_K);           
                
                selfnum=sum(Sc);
                selfnum_KA=sum(Sc_KA);
                selfnum_LA=sum(Sc_LA);
                selfnum_SLK=sum(Sc_SLK);
                
                Rem=zeros(n,1);
                                
                
                Matched_p=zeros(n,1);
                Ns=[];
                for num_per=1:max_per;
                    
                    if mod(num_per,10)==0
                        fprintf('\n');
                        
                    end;
                    
                    Rem(Arr_KA(num_per):Arr_KA(num_per+1)-1,1)=1;
                    Rem(Arr_LA(num_per):Arr_LA(num_per+1)-1,1)=1;
                    Rem(Arr_SLK(num_per):Arr_SLK(num_per+1)-1,1)=1;
                    Rem=Rem&(~Sc);
                   
                    
                    Selfnum_KA(num_sim,num_per)=sum(Sc_KA(Arr_KA(num_per):Arr_KA(num_per+1)-1,1));
                    Selfnum_LA(num_sim,num_per)=sum(Sc_LA(Arr_LA(num_per):Arr_LA(num_per+1)-1,1));
                    Selfnum_SLK(num_sim,num_per)=sum(Sc_SLK(Arr_SLK(num_per):Arr_SLK(num_per+1)-1,1));
                    
                    Rem_KA=(Rem>0)&(Pats<=n_KA);
                    Rem_SLK=(Rem>0)&((Pats>n_KA)&(Pats<=n_K));
                    Rem_LA=(Rem>0)&(Pats>n_K);
                    
                    Patnum_KA(num_sim,num_per)=sum(Rem_KA);
                    Patnum_LA(num_sim,num_per)=sum(Rem_LA);
                    Patnum_SLK(num_sim,num_per)=sum(Rem_SLK);
                    
                    
                    Pats_KA_ns=Pats(Rem_KA>0);
                    
                    Pats_SLK_ns=Pats(Rem_SLK>0);
                    
                    Pats_LA_ns=Pats(Rem_LA>0);
                    
                    
                    Comp_K_ns=Comp_K((Rem),(Rem)');
                    
                    Comp_L_ns=Comp_L((Rem),(Rem)');
                    
                    
                    n_ns=size(Comp_L_ns,1);
                    
                    
                    Same_pat=[];
                    NDD=[];
                    gen_num=1;
                    
                    Ept=zeros(1,4);
                    Apt=zeros(1,4);
                    Spt=zeros(1,4);
                    
                    
                    XNDD=zeros(n_ns,1);
                    gen_num=1;
                    
                    
                    Comp_K_KA_ns=Comp_K(Rem_KA,Rem_KA');
                    Comp_L_KA_ns=Comp_L(Rem_KA,Rem_KA');
                    
                    Comp_K_SLK_ns=Comp_K(Rem_SLK,Rem_SLK');
                    Comp_L_SLK_ns=Comp_L(Rem_SLK,Rem_SLK');
                    
                    Comp_L_LA_ns=Comp_L(Rem_LA,Rem_LA');
                    Comp_K_LA_ns=Comp_K(Rem_LA,Rem_LA');
                    
                    n_KA_ns=size(Comp_K_KA_ns,1);
                    n_SLK_ns=size(Comp_K_SLK_ns,1);
                    n_LA_ns=size(Comp_K_LA_ns,1);
                    
                    We_ns=ones(n_ns,1);
                    We_KA_ns=ones(n_KA_ns,1);
                    We_SLK_ns=ones(n_SLK_ns,1);
                    We_LA_ns=ones(n_LA_ns,1);
                    
                    
                   
                    Matched=zeros(n_ns,1);
                    Matched_KA=zeros(n_KA_ns,1);
                    Matched_SLK=zeros(n_SLK_ns,1);
                    Matched_LA=zeros(n_LA_ns,1);
                    diary off;
                    if num_treat==1
                        
                        [num_cyc_KA,Cyc_size_KA,Cycles_KA,In_cyc_KA,W_cyc_KA]=...
                            wfind_cyc_size2_SLK_exchange(n_KA_ns,0,0,Comp_K_KA_ns,Comp_L_KA_ns,We_KA_ns);
                        
                        [num_cyc_SLK,Cyc_size_SLK,Cycles_SLK,In_cyc_SLK,W_cyc_SLK]=...
                            wfind_cyc_size2_SLK_exchange(0,n_SLK_ns,0,Comp_K_SLK_ns,Comp_L_SLK_ns,We_SLK_ns);
                        [num_cyc_LA,Cyc_size_LA,Cycles_LA,In_cyc_LA,W_cyc_LA]=...
                            wfind_cyc_size2_SLK_exchange(0,0,n_LA_ns,Comp_K_LA_ns,Comp_L_LA_ns,We_LA_ns);
                        
                        %fprintf('Calculating 2-way Optimized Match ...');
                        
                        [Matched_KA,exnum_KA,Mu_KA]=...
                            wMAX_K_WAY_SLK_exchange(n_KA_ns,Cyc_size_KA,Cycles_KA,In_cyc_KA,...
                            W_cyc_KA,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                        [Matched_SLK,exnum_SLK,Mu_SLK]=...
                            wMAX_K_WAY_SLK_exchange(n_SLK_ns,Cyc_size_SLK,Cycles_SLK,In_cyc_SLK,...
                            W_cyc_SLK,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                        
                        [Matched_LA,exnum_LA,Mu_LA]=...
                            wMAX_K_WAY_SLK_exchange(n_LA_ns,Cyc_size_LA,Cycles_LA,In_cyc_LA,...
                            W_cyc_LA,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                        %exnum=exnum_KA+exnum_LA+exnum_SLK;
                    elseif num_treat==2
                        
                        
                        [num_cyc,Cyc_size,Cycles,In_cyc,W_cyc]=...
                            wfind_cyc_size2_SLK_exchange(n_KA_ns,n_SLK_ns,n_LA_ns,Comp_K_ns,Comp_L_ns,We_ns);
                        
                        %fprintf('Calculating 2-way Optimized Match ...');
                        [Matched,exnum,Mu]=...
                            wMAX_K_WAY_SLK_exchange(n_ns,Cyc_size,Cycles,In_cyc,...
                            W_cyc,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                        %fprintf('Done \n');
                        Matched_KA=Matched(1:n_KA_ns);
                        Matched_SLK=Matched(n_KA_ns+1:n_KA_ns+n_SLK_ns);
                        Matched_LA=Matched(n_KA_ns+n_SLK_ns+1:n_ns);
                        
                        exnum_KA=sum(Matched_KA);
                        exnum_SLK=sum(Matched_SLK);
                        exnum_LA=sum(Matched_LA);
                        
                    end;
                    Exnum_SLK(num_sim, num_per)=exnum_SLK;
                    Exnum_KA(num_sim,num_per)=exnum_KA;
                    Exnum_LA(num_sim,num_per)=exnum_LA;
                    Matched_p=zeros(n,1);
                    [Icoor,Bogs]=find(Matched_KA);
                    n_I=size(Icoor,1);
                    for ii=1:n_I;
                        p=Pats_KA_ns(Icoor(ii));
                        Matched_p(p)=1;
                    end;
                    [Icoor,Bogs]=find(Matched_SLK);
                    n_I=size(Icoor,1);
                    for ii=1:n_I;
                        p=Pats_SLK_ns(Icoor(ii));
                        Matched_p(p)=1;
                    end;
                    
                    [Icoor,Bogs]=find(Matched_LA);
                    n_I=size(Icoor,1);
                    for ii=1:n_I;
                        p=Pats_LA_ns(Icoor(ii));
                        Matched_p(p)=1;
                    end;
                    %find(Matched_p)
                    %pause
                    
                    Rem=Rem &(~Matched_p);
                   
                    t=toc;
                    
                    
                    fprintf('%8.2f ', t);
                    Ns=[Ns;n_KA_ns n_SLK_ns n_LA_ns Selfnum_KA(num_sim, num_per) Selfnum_SLK(num_sim, num_per) Selfnum_LA(num_sim,num_per)  Exnum_KA(num_sim, num_per) Exnum_SLK(num_sim, num_per) Exnum_LA(num_sim,num_per)];
                end; %num_per
                tic;
               
                if data_all
                    if num_treat==2
                        CK_a=Comp_K(~Sc,(~Sc)');
                        CL_a=Comp_L(~Sc,(~Sc)');
                        n_a=size(CK_a,1);
                        n_KA_a=n_KA-sum(Sc_KA);
                        n_LA_a=n_LA-sum(Sc_LA);
                        n_SLK_a=n_SLK-sum(Sc_SLK);
                        We_a=ones(n_a,1);
                        [num_cyc,Cyc_size,Cycles,In_cyc,W_cyc]=...
                            wfind_cyc_size2_SLK_exchange(n_KA_a,n_SLK_a,n_LA_a,CK_a,CL_a,We_a);
                        
                        %fprintf('Calculating 2-way Optimized Match ...');
                        [M_a,exnum_a,Mu_a]=...
                            wMAX_K_WAY_SLK_exchange(n_a,Cyc_size,Cycles,In_cyc,...
                            W_cyc,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                        %fprintf('Done \n');
                        M_KA_a=M_a(1:n_KA_a);
                        M_SLK_a=M_a(n_KA_a+1:n_KA_a+n_SLK_a);
                        M_LA_a=M_a(n_KA_a+n_SLK_a+1:n_a);
                        
                        exnum_KA_a=sum(M_KA_a);
                        exnum_SLK_a=sum(M_SLK_a);
                        exnum_LA_a=sum(M_LA_a);
                        
                        
                    else
                        CK_KA_a=Comp_K(~Sc_KA,(~Sc_KA)');
                        CL_KA_a=Comp_L(~Sc_KA,(~Sc_KA)');
                        CK_SLK_a=Comp_K(~Sc_SLK,(~Sc_SLK)');
                        CL_SLK_a=Comp_L(~Sc_SLK,(~Sc_SLK)');
                        CK_LA_a=Comp_K(~Sc_LA,(~Sc_LA)');
                        CL_LA_a=Comp_L(~Sc_LA,(~Sc_LA)');
                        n_KA_a=size(CK_KA_a,1);
                        n_LA_a=size(CK_LA_a,1);
                        n_SLK_a=size(CK_SLK_a,1);
                        We_KA_a=ones(n_KA_a,1);
                        We_SLK_a=ones(n_SLK_a,1);
                        We_LA_a=ones(n_LA_a,1);
                        [num_cyc_KA,Cyc_size_KA,Cycles_KA,In_cyc_KA,W_cyc_KA]=...
                            wfind_cyc_size2_SLK_exchange(n_KA_a,0,0,CK_KA_a,CL_KA_a,We_KA_a);
                        
                        [num_cyc_SLK,Cyc_size_SLK,Cycles_SLK,In_cyc_SLK,W_cyc_SLK]=...
                            wfind_cyc_size2_SLK_exchange(0,n_SLK_a,0,CK_SLK_a,CL_SLK_a,We_SLK_a);
                        [num_cyc_LA,Cyc_size_LA,Cycles_LA,In_cyc_LA,W_cyc_LA]=...
                            wfind_cyc_size2_SLK_exchange(0,0,n_LA_a,CK_LA_a,CL_LA_a,We_LA_a);
                        
                        
                        %fprintf('Calculating 2-way Optimized Match ...');
                        
                        [M_KA_a,exnum_KA_a,Mu_KA_a]=...
                            wMAX_K_WAY_SLK_exchange(n_KA_a,Cyc_size_KA,Cycles_KA,In_cyc_KA,...
                            W_cyc_KA,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                        [M_SLK_a,exnum_SLK_a,Mu_SLK_a]=...
                            wMAX_K_WAY_SLK_exchange(n_SLK_a,Cyc_size_SLK,Cycles_SLK,In_cyc_SLK,...
                            W_cyc_SLK,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                        
                        [M_LA_a,exnum_LA_a,Mu_LA_a]=...
                            wMAX_K_WAY_SLK_exchange(n_LA_a,Cyc_size_LA,Cycles_LA,In_cyc_LA,...
                            W_cyc_LA,num_cyc_size,Same_pat,gen_num,weight_true,cplex_true);
                        exnum_a=exnum_KA_a+exnum_LA_a+exnum_SLK_a;
                        
                        
                    end
                    t=toc;
                    fprintf('Static Run done, time: %10.2f', t);
                    
                end %data_all
                fprintf('\n');
                
                
                exnum_KA_t=sum((Exnum_KA(num_sim,:))');
                exnum_LA_t=sum((Exnum_LA(num_sim,:))');
                exnum_SLK_t=sum((Exnum_SLK(num_sim,:))');
                exnum_t=exnum_KA_t+exnum_LA_t+exnum_SLK_t;
                diary on;
                results=[num_pop num_perc num_treat num_sim -1 n n_KA n_SLK n_LA  -2 ...
                    selfnum selfnum_KA selfnum_SLK selfnum_LA -3 exnum_t exnum_KA_t ...
                    exnum_SLK_t exnum_LA_t -4];
                if data_all
                    results=[results exnum_a exnum_KA_a exnum_SLK_a exnum_LA_a -5];
                end;
                results
                diary off;
                
                results=[results   mean(Plist) std(Plist)];
                
                
                stats=[stats;results];
            end; %num_sim
            
            stats=[stats  Sep Patnum_KA Sep Patnum_SLK Sep Patnum_LA 2*Sep Selfnum_KA 2*Sep Selfnum_SLK 2*Sep Selfnum_LA 3*Sep Exnum_KA 3*Sep Exnum_SLK 3*Sep Exnum_LA];
            
            str1=int2str(n);
            
            str2=int2str(num_treat);
            
            
            outfilename=strcat(fnamestr,'ALL-',dstr,'-',mcsstr,'-',str1,'-',str2,'-',str3,'.csv');
            outfilename2 =strcat(fnamestr,'AVG-',dstr,'-',mcsstr,'-',str1,'-',str2,'-',str3,'.csv');
            outfilename3 =strcat(fnamestr,'STD-',dstr,'-',mcsstr,'-',str1,'-',str2,'-',str3,'.csv');
            
            csvwrite(outfilename,stats);
            %outfilename
            ansavr=[ansavr;mean(stats)]
            ansstd=[ansstd;std(stats)/denomstd];
            csvwrite(outfilename2,ansavr);
            csvwrite(outfilename3,ansstd);
        end; %treat_num
    end; %perc_num
end; %pop_num

diary off;