%
% DUAL-GRAFT LIVER EXCHANGE SIMULATIONS
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% k-way maximization function for single-graft liver exchange for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%

function [Mu_x,eff_x]=...
    wMAX_K_WAY_one_lobe_liver_exchange(n,Comp, We, num, ...
    Cyc_size,Cycles,In_cyc,W_cyc,num_cyc_size,...
    Same_pat,gen_num,weight_true,cplex_true);
Mu_x=zeros(n,gen_num);
Matched_x=zeros(n,gen_num);

eff_x=0;
maxw_x=[];
Cycles_x=[];
In_x = Cyc_size <= num_cyc_size;
Cycles_x=Cycles(In_x,:);
Cyc_size_x=Cyc_size(In_x);
In_cyc_x=In_cyc(In_x,:);
W_cyc_x=W_cyc(In_x);
for ss=1:size(Same_pat,1);
    pat1=Same_pat(ss,1);
    pat2=Same_pat(ss,2);
    a1=pat1==Cycles_x(:,1);
    a2=pat1==Cycles_x(:,2);
    a3=pat1==Cycles_x(:,3);
    a4=pat1==Cycles_x(:,4);
    b1=pat2==Cycles_x(:,1);
    b2=pat2==Cycles_x(:,2);
    b3=pat2==Cycles_x(:,3);
    b4=pat2==Cycles_x(:,4);
    ac=a1|a2|a3|a4;
    bc=b1|b2|b3|b4;
    kk=~(ac&bc);
    Cycles_x=Cycles_x(kk>0,:);
    In_cyc_x=In_cyc_x(kk>0,:);
    W_cyc_x=W_cyc_x(kk>0,:);
    Cyc_size_x=Cyc_size_x(kk>0,:);
end;
num_x=size(In_cyc_x,1);
xU=ones(num_x,1);
xL=zeros(num_x,1);
f=-Cyc_size_x;
w=-W_cyc_x;
A=In_cyc_x';
bU=ones(n,1);
bL=zeros(n,1);
for ss=1:size(Same_pat,1);
    pat1=Same_pat(ss,1);
    pat2=Same_pat(ss,2);
    a1=pat1==Cycles_x(:,1);
    a2=pat1==Cycles_x(:,2);
    a3=pat1==Cycles_x(:,3);
    a4=pat1==Cycles_x(:,4);
    b1=pat2==Cycles_x(:,1);
    b2=pat2==Cycles_x(:,2);
    b3=pat2==Cycles_x(:,3);
    b4=pat2==Cycles_x(:,4);
    ac=a1|a2|a3|a4;
    bc=b1|b2|b3|b4;
    ccs=(ac|bc);
    A=[A;ccs'];
    bL=[bL;0];
    bU=[bU;1];
end;

if num_x>0
    if ~cplex_true
        an=size(A,1);
        CTYPE=zeros(an,1);
        CTYPE(1:an,1)='U';
        
        CTYPE=char(CTYPE);
        
        VARTYPES=zeros(num_x,1);
        VARTYPES(1:num_x,1)='I';
        VARTYPES=char(VARTYPES);
        [X1,FOPT,LPSOLVED,STATUS]=glpk(f,A,bU,xL,xU,CTYPE,VARTYPES,1);
    else
        VARTYPES2=zeros(1,num_x);
        VARTYPES2(1,1:num_x)='B';
       
        VARTYPES2=char(VARTYPES2);
        [X1,FOPT,LPSOLVED,STATUS]= cplexmilp(f,A,bU,[],[],[],[],[],xL,xU,VARTYPES2);
    end;
    X2=round(X1);
    fmin=f'*X2;
    eff_x=-fmin;
    if weight_true
        A=[A;f'];
        bL=[bL;fmin];
        bU=[bU;fmin];
        X2=[];
        
        
        for gg=1:gen_num
            if ~cplex_true
                CTYPE=[CTYPE;'S'];
                [X,FOPT,LPSOLVED,STATUS]=glpk(w,A,bU,xL,xU,CTYPE,VARTYPES,1);
            else
                X=cplexmilp(w,A,bU,[],[],[],[],[],xL,xU,VARTYPES2);
            end;
            X=round(X);
            whi=sum(X);
            X2=[X2 X];
            In_x=[In_x;X'];
            bL=[bL;0];
            bU=[bU;whi-1];
            CTYPE=[CTYPE;'U'];
            maxw_x(gg)=-w'*X;
        end;
       
    end;
   
    Mu_x=zeros(n,gen_num);
    
    for gg=1:gen_num
        for cc=1:num_x
            if X2(cc,gg)
                for kk=1:Cyc_size_x(cc)-1
                    pat=Cycles_x(cc,kk);
                    don=Cycles_x(cc,kk+1);
                    Matched_x(pat,gg)=1;
                    Mu_x(pat,gg)=don;
                    
                end;
                Mu_x(Cycles_x(cc,Cyc_size_x(cc)),gg)=Cycles_x(cc,1);
                
            end;
        end;
    end;
   
end;