%
% DUAL-GRAFT LIVER EXCHANGE SIMULATIONS
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% unrestricted maximization function for dual-graft liver exchange for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
function  [Matchedtrue, num_matched, Mu, E_inout, E_out, E_donor, Edges, C_f, X]=...
    wMAX_UNR_dual_lobe_liver_exchange(ABOc, Livd, Livp, threshold, cplex_true)

n=size(ABOc,1);
Mu=zeros(n,2);
Matchedtrue=zeros(n,1);
num_matched=0;
n_d=2*n;

C_f=zeros(n,n_d^2);

for p=1:n
    for d1=1:n_d
        
        for d2=d1+1:n_d
            edge=(d1-1)*n_d+d2;   
            
            C_f(p,edge)=(ABOc(p,d1)*ABOc(p,d2))*((Livd(d1)+Livd(d2))>=Livp(p)*threshold);
        end;
    end;
end;

E_inout=[];
E_donor=[];
E_out=[];
xU=[];
Edges=[];
X=[];
[i_list,e_list]=find(C_f);
if ~isempty(i_list)    
   
    max_edge=size(i_list,1);
    E_inout=zeros(n,max_edge);
    E_donor=zeros(n,max_edge);
    E_out=zeros(n,max_edge);
    xU=ones(max_edge,1);
    Edges=[];
    f=-ones(max_edge,1);
    
    e1=ones(n,1);
    e0=zeros(n,1);
        
    for e_num=1:max_edge
        p=i_list(e_num);       
        E_inout(p,e_num)=E_inout(p,e_num)+1;
        E_out(p,e_num)=1;
        edge=e_list(e_num);
        
        %xU(enum)=1;
        
        d1=floor((edge-1)/n_d)+1;
        d2=edge-(d1-1)*n_d;
        
        p_d2=floor((d2-1)/2)+1;
        p_d1=floor((d1-1)/2)+1;
        if p_d1~=p_d2
            if mod(d1,2)==1
                E_donor(p_d1,e_num)=1;
            elseif mod(d1,2)==0
                E_donor(p_d1,e_num)=-1;
            end;
            if mod(d2,2)==1
                E_donor(p_d2,e_num)=1;
            elseif mod(d2,2)==0
                E_donor(p_d2,e_num)=-1;
            end;
        end;
        Edges=[Edges; p edge d1 d2 p_d1 p_d2  E_donor(p_d1,e_num)  E_donor(p_d2,e_num)];
        E_inout(p_d1,e_num)=E_inout(p_d1,e_num)-0.5;
        E_inout(p_d2,e_num)=E_inout(p_d2,e_num)-0.5;
    end;
    
    xL=zeros(max_edge,1);
    bL=zeros(3*n,1);
    bU=[e0;e1;e0];
    A=[E_inout;E_out;E_donor];
    
    
    
    if ~isempty(A)
        if ~cplex_true
            
            CTYPE=zeros(2*n,1);
            
            CTYPE(1:n,1)='S';
            CTYPE(n+1:2*n,1)='U';
            CTYPE(2*n+1:3*n,1)='S';
            
            VARTYPES=zeros(max_edge,1);
            VARTYPES(1:max_edge,1)='I';
            
            VARTYPES=char(VARTYPES);
            CTYPE=char(CTYPE);
            
            [X,FOPT,LPSOLVED,STATUS]=glpk(f,A,bU,xL,xU,CTYPE,VARTYPES,1);
        else
            Aeq=[E_inout;E_donor];
            Aineq=E_out;
            beq=[e0;e0];
            bup=e1;
            VARTYPES=zeros(1,max_edge);
            VARTYPES(1,1:max_edge)='B';
            
            VARTYPES=char(VARTYPES);
            [X,FOPT,LPSOLVED,STATUS]= cplexmilp(f,Aineq,bup,Aeq,beq,[],[],[],xL,xU,VARTYPES);
        end;
        X=round(X);
        
        d=zeros(2,1);
        
        for e_num=1:max_edge
            if X(e_num)==1
                edge=e_list(e_num);
                d(1)=floor((edge-1)/n_d)+1;           
                d(2)=edge-(d(1)-1)*n_d;
                for ii=1:2
                    Mu(i_list(e_num),ii)=round(d(ii)/2);
                end;
                Matchedtrue(i_list(e_num))=1;
            end;
        end;
        num_matched=sum(Matchedtrue);
        %muxm=[muxm mu1];
        %mu1
        % pause
        %   optdone=optdone|(gg==gennum);
        %end;
        %SOLNxm=[];
        %cycxm=[];
        %mxncyclen=0;
        %wexm=[];
        %for gg=1:matchnum
        %   done=zeros(n,1);
        %   mu1=muxm(:,gg);
        %    cyc1=[];
        %    we1=[];
        %    for pat=1:n;
        %        if ~done(pat)
        %            if mu1(pat)~=0
        %                cyc=pat;
        %                done(pat)=1;
        %                kk=mu1(pat);
        %                we=We(pat,kk);
        %                nn=1;
        %                while ~done(kk)
        %                    cyc=[cyc kk];
        %                    done(kk)=1;
        %                    we=[we We(kk,mu1(kk))];
        %                    kk=mu1(kk);
        %                    nn=nn+1;
        %                end;
        %                mxncyclen=max([nn;mxncyclen]);
        %                cyc1=[cyc1; cyc zeros(1, n-size(cyc,2))];
        %                we1=[we1; we zeros(1, n-size(cyc,2))];
        %            else
        %                done(pat)=1;
        %            end;
        %        end;
        %    end;
        %    SOLN1=[zeros(size(SOLNxm,1),1);ones(size(cyc1,1),1)];
        %    SOLNxm=[SOLNxm;zeros(size(cyc1,1),size(SOLNxm,2))];
        %    SOLNxm=[SOLNxm SOLN1];
        %    cycxm=[cycxm;cyc1];
        %    wexm=[wexm;we1];
    end;
end;
%end;