
%
% DUAL-GRAFT LIVER EXCHANGE SIMULATIONS
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% Function to find all cycles for dual-graft liver exchange for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
% uses CPLEX of IBM
%

function [num_cyc, Cyc_size, Cycles, In_cyc, W_cyc]=...
    wfind_cyc_sizeupto5_dual_lobe_liver_exchange(ABOcd1,ABOcd2,Livd,Livp,threshold,num_cyc_size,cplex_true);

n=size(ABOcd1,1);

maxc=10000;
Cyc_size=zeros(maxc,1);
Cycles=zeros(maxc,5);
In_cyc=zeros(maxc,n);

num_cyc=0;
for i=1:n
    
    for j=i+1:n
        size_c=2;
        ABOc_c=zeros(size_c,2*size_c);
        Cycp=[i;j];
        Cycd=[2*i-1;2*i;2*j-1;2*j];
        
        for ii=1:size_c
            p=Cycp(ii);
            ABOc_c(:,2*ii-1:2*ii)=[ABOcd1(Cycp,p) ABOcd2(Cycp,p)];
        end;
        
        Livd_c=Livd(Cycd);
        Livp_c=Livp(Cycp);
       
        [Matchedtrue, num_matched, Mu, E_inout, E_out, E_donor, Edges, C_f, X]=...
            wMAX_UNR_dual_lobe_liver_exchange(ABOc_c, Livd_c, Livp_c, threshold, cplex_true);
        if num_matched==size_c
            num_cyc=num_cyc+1;
            Cycles(num_cyc,:)=[i j 0 0 0];
            
            Cyc_size(num_cyc)=size_c;
            a=zeros(1,n);
            a(i)=1;
            a(j)=1;
            In_cyc(num_cyc,:)=a;
        end;
        if num_cyc_size>2
            
            for k=j+1:n
                size_c=3;
                ABOc_c=zeros(size_c,2*size_c);
                Cycp=[i;j;k];
                Cycd=[2*(i-1)+1;2*i;2*(j-1)+1;2*j;2*(k-1)+1;2*k];
                
                for ii=1:size_c
                    p=Cycp(ii);
                    ABOc_c(:,2*ii-1:2*ii)=[ABOcd1(Cycp,p) ABOcd2(Cycp,p)];
                    
                end;
                
                Livd_c=Livd(Cycd);
                Livp_c=Livp(Cycp);
               
                [Matchedtrue, num_matched, Mu, E_inout, E_out, E_donor, Edges, C_f, X]=...
                    wMAX_UNR_dual_lobe_liver_exchange(ABOc_c, Livd, Livp, threshold, cplex_true);
                if num_matched==size_c
                    num_cyc=num_cyc+1;
                    Cycles(num_cyc,:)=[i j k 0 0];
                   
                    Cyc_size(num_cyc)=size_c;
                    a=zeros(1,n);
                    a(i)=1;
                    a(j)=1;
                    a(k)=1;
                    In_cyc(num_cyc,:)=a;
                end;
                if num_cyc_size>3
                    
                    for m=k+1:n
                        size_c=4;
                        ABOc_c=zeros(size_c,2*size_c);
                        Cycp=[i;j;k;m];
                        Cycd=[2*(i-1)+1;2*i;2*(j-1)+1;2*j;2*(k-1)+1;2*k;2*(m-1)+1;2*m];
                        
                        for ii=1:size_c
                            p=Cycp(ii);
                            ABOc_c(:,2*ii-1:2*ii)=[ABOcd1(Cycp,p) ABOcd2(Cycp,p)];
                        end;
                        
                        Livd_c=Livd(Cycd);
                        Livp_c=Livp(Cycp);
                        
                        [Matchedtrue, num_matched, Mu, E_inout, E_out, E_donor, Edges, C_f, X]=...
                            wMAX_UNR_dual_lobe_liver_exchange(ABOc_c, Livd_c, Livp_c, threshold, cplex_true);
                        if num_matched==size_c
                            num_cyc=num_cyc+1;
                            Cycles(num_cyc,:)=[i j k m 0];
                            
                            Cyc_size(num_cyc)=size_c;
                            
                            a=zeros(1,n);
                            a(i)=1;
                            a(j)=1;
                            a(k)=1;
                            a(m)=1;
                            
                            In_cyc(num_cyc,:)=a;
                        end;
                        if num_cyc_size>4
                            
                            for o=m+1:n
                                size_c=5;
                                ABOc_c=zeros(size_c,2*size_c);
                                Cycp=[i;j;k;m;o];
                                Cycd=[2*(i-1)+1;2*i;2*(j-1)+1;2*j;2*(k-1)+1;2*k;2*(m-1)+1;2*m;2*(o-1)+1;2*o];
                                
                                for ii=1:size_c
                                    p=Cycp(ii);
                                    ABOc_c(:,2*ii-1:2*ii)=[ABOcd1(Cycp,p) ABOcd2(Cycp,p)];
                                end;
                                Livd_c=Livd(Cycd);
                                Livp_c=Livp(Cycp);
                                
                                [Matchedtrue, num_matched, Mu, E_inout, E_out, E_donor, Edges, C_f, X]=...
                                    wMAX_UNR_dual_lobe_liver_exchange(ABOc_c, Livd_c, Livp_c, threshold, cplex_true);
                                if num_matched==size_c
                                    num_cyc=num_cyc+1;
                                    Cycles(num_cyc,:)=[i j k m o];
                                    
                                    Cyc_size(num_cyc)=size_c;
                                    
                                    a=zeros(1,n);
                                    a(i)=1;
                                    a(j)=1;
                                    a(k)=1;
                                    a(m)=1;
                                    a(o)=1;
                                    In_cyc(num_cyc,:)=a;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;
Cycles=Cycles(1:num_cyc,:);
In_cyc=In_cyc(1:num_cyc,:);
Cyc_size=Cyc_size(1:num_cyc,:);
W_cyc=Cyc_size;