
%
% LUNG EXCHANGE SIMULATIONS
% 
% Multi-Donor Organ Exchange 
% by Haluk Ergin, Tayfun Sonmez, and M. Utku Unver
% 
% Part of Supplementary Appendix
% 
% Function to find all cycles for lung exchange  for MATLAB
% (c) Ergin, Sonmez, Unver 2014
%
% uses assignmentoptimal.m downloaded from 
% https://www.mathworks.com/matlabcentral/fileexchange/6543-functions-for-the-rectangular-assignment-problem/content/assignmentoptimal.m?requestedDomain=www.mathworks.com
% for Hungarian method in bi-partite cardinality matching
% which is current as of 01/07/2017
%
function [num_cyc, Cyc_size, Cycles, In_cyc, W_cyc]=...
    wfind_cyc_sizeupto5_lung_exchange_fast(n_ns,Comp1_ns,Comp2_ns,We,num_cyc_size)
Mxxx=10000*n_ns;
maxdim=n_ns*1000;
In_cyc=zeros(maxdim,n_ns);
Cyc_size=zeros(maxdim,1);
Cycles=zeros(maxdim,5);
%In_cyc=[];
%W_cyc=zeros(maxdim,1);
Cmult_2=Mxxx*ones(4,4);
Cmult_3=Mxxx*ones(6,6);
Cmult_4=Mxxx*ones(8,8);
Cmult_5=Mxxx*ones(10,10);
num_cyc=0;
for i=1:n_ns
    for j=i+1:n_ns
        Ci=[Comp1_ns(i,i) Comp2_ns(i,i) Comp1_ns(i,j) Comp2_ns(i,j)];
        Cj=[Comp1_ns(j,i) Comp2_ns(j,i) Comp1_ns(j,j) Comp2_ns(j,j)];
        C2=[Ci;Ci;Cj;Cj];
        C=C2+(Cmult_2.*(1-C2));
        [mu,num_matched_edge]=assignmentoptimal(C);
        if num_matched_edge==4
            num_cyc=num_cyc+1;
            Cycles(num_cyc,:)=[i j 0 0 0];
            %W_cyc=[W_cyc;weight(i)+weight(j)];
            Cyc_size(num_cyc)=2;
            a=zeros(1,n_ns);
            a(i)=1;
            a(j)=1;
            In_cyc(num_cyc,:)=a;
        end;
        if num_cyc_size>2
            for k=j+1:n_ns
                Ci=[Comp1_ns(i,i) Comp2_ns(i,i) Comp1_ns(i,j) Comp2_ns(i,j) Comp1_ns(i,k) Comp2_ns(i,k)];
                Cj=[Comp1_ns(j,i) Comp2_ns(j,i) Comp1_ns(j,j) Comp2_ns(j,j) Comp1_ns(j,k) Comp2_ns(j,k)];
                Ck=[Comp1_ns(k,i) Comp2_ns(k,i) Comp1_ns(k,j) Comp2_ns(k,j) Comp1_ns(k,k) Comp2_ns(k,k)];
                C2=[Ci;Ci;Cj;Cj;Ck;Ck];
                C=C2+(Cmult_3.*(1-C2));
                [mu,num_matched_edge]=assignmentoptimal(C);
                if num_matched_edge==6
                    num_cyc=num_cyc+1;
                    Cycles(num_cyc,:)=[i j k 0 0];
                    %W_cyc=[W_cyc;weight(i)+weight(j)];
                    Cyc_size(num_cyc)=3;
                    a=zeros(1,n_ns);
                    a(i)=1;
                    a(j)=1;
                    a(k)=1;
                    In_cyc(num_cyc,:)=a;
                    
                end;
                if num_cyc_size>3
                    for m=k+1:n_ns
                        
                        Ci=[Comp1_ns(i,i) Comp2_ns(i,i) Comp1_ns(i,j) ...
                            Comp2_ns(i,j) Comp1_ns(i,k) Comp2_ns(i,k) Comp1_ns(i,m) Comp2_ns(i,m)];
                        Cj=[Comp1_ns(j,i) Comp2_ns(j,i) Comp1_ns(j,j) ...
                            Comp2_ns(j,j) Comp1_ns(j,k) Comp2_ns(j,k) Comp1_ns(j,m) Comp2_ns(j,m)];
                        Ck=[Comp1_ns(k,i) Comp2_ns(k,i) Comp1_ns(k,j) ...
                            Comp2_ns(k,j) Comp1_ns(k,k) Comp2_ns(k,k) Comp1_ns(k,m) Comp2_ns(k,m)];
                        Cm=[Comp1_ns(m,i) Comp2_ns(m,i) Comp1_ns(m,j) ...
                            Comp2_ns(m,j) Comp1_ns(m,k) Comp2_ns(m,k) Comp1_ns(m,m) Comp2_ns(m,m)];
                        C2=[Ci;Ci;Cj;Cj;Ck;Ck;Cm;Cm];
                        
                        C=C2+(Cmult_4.*(1-C2));
     
                        [mu,num_matched_edge]=assignmentoptimal(C);
                        
                        %end;
                        if num_matched_edge==8
                            num_cyc=num_cyc+1;
                            Cycles(num_cyc,:)=[i j k m 0];
                            %W_cyc=[W_cyc;weight(i)+weight(j)];
                            Cyc_size(num_cyc)=4;
                            a=zeros(1,n_ns);
                            a(i)=1;
                            a(j)=1;
                            a(k)=1;
                            a(m)=1;
                            In_cyc(num_cyc,:)=a;
                        end;
                        if num_cyc_size>4
                            for p=m+1:n_ns
                                Ci=[Comp1_ns(i,i) Comp2_ns(i,i) Comp1_ns(i,j) ...
                                    Comp2_ns(i,j) Comp1_ns(i,k) Comp2_ns(i,k) ...
                                    Comp1_ns(i,m) Comp2_ns(i,m)...
                                    Comp1_ns(i,p) Comp2_ns(i,p)];
                                Cj=[Comp1_ns(j,i) Comp2_ns(j,i) Comp1_ns(j,j) ...
                                    Comp2_ns(j,j) Comp1_ns(j,k) Comp2_ns(j,k)...
                                    Comp1_ns(j,m) Comp2_ns(j,m)...
                                    Comp1_ns(j,p) Comp2_ns(j,p)];
                                Ck=[Comp1_ns(k,i) Comp2_ns(k,i) Comp1_ns(k,j) ...
                                    Comp2_ns(k,j) Comp1_ns(k,k) Comp2_ns(k,k) ...
                                    Comp1_ns(k,m) Comp2_ns(k,m)...
                                    Comp1_ns(k,p) Comp2_ns(k,p)];
                                Cm=[Comp1_ns(m,i) Comp2_ns(m,i) Comp1_ns(m,j) ...
                                    Comp2_ns(m,j) Comp1_ns(m,k) Comp2_ns(m,k) ...
                                    Comp1_ns(m,m) Comp2_ns(m,m)...
                                    Comp1_ns(m,p) Comp2_ns(m,p)];
                                Cp=[Comp1_ns(p,i) Comp2_ns(p,i) Comp1_ns(p,j) ...
                                    Comp2_ns(p,j) Comp1_ns(p,k) Comp2_ns(p,k) ...
                                    Comp1_ns(p,m) Comp2_ns(p,m)...
                                    Comp1_ns(p,p) Comp2_ns(p,p)];
                                C2=[Ci;Ci;Cj;Cj;Ck;Ck;Cm;Cm;Cp;Cp];
                                
                                C=C2+(Cmult_5.*(1-C2));
                                [mu,num_matched_edge]=assignmentoptimal(C);
                                if num_matched_edge==10
                                    num_cyc=num_cyc+1;
                                    Cycles(num_cyc,:)=[i j k m p];
                                    %W_cyc=[W_cyc;weight(i)+weight(j)];
                                    Cyc_size(num_cyc)=5;
                                    a=zeros(1,n_ns);
                                    a(i)=1;
                                    a(j)=1;
                                    a(k)=1;
                                    a(m)=1;
                                    a(p)=1;
                                    In_cyc(num_cyc,:)=a;
                                end;
                            end;
                        end;
                    end;
                end;
            end;
        end;
    end;
end;

In_cyc=In_cyc(1:num_cyc,:);
Cycles=Cycles(1:num_cyc,:);
Cyc_size=Cyc_size(1:num_cyc,:);
W_cyc=Cyc_size;